# Generated by roxygen2: do not edit by hand

export(cla_id_paramPos)
export(cla_secsse_loglik)
export(cla_secsse_ml)
export(cla_secsse_ml_func_def_pars)
export(create_default_lambda_transition_matrix)
export(create_default_shift_matrix)
export(create_lambda_list)
export(create_mu_vector)
export(create_q_matrix)
export(default_params_doc)
export(event_times)
export(expand_q_matrix)
export(extract_par_vals)
export(fill_in)
export(id_paramPos)
export(plot_idparslist)
export(plot_state_exact)
export(prepare_full_lambdas)
export(q_doubletrans)
export(secsse_loglik)
export(secsse_loglik_eval)
export(secsse_ml)
export(secsse_ml_func_def_pars)
export(secsse_sim)
export(secsse_single_branch_loglik)
export(sortingtraits)
import(Rcpp)
importFrom(RcppParallel, RcppParallelLibs)
importFrom(rlang,.data)
useDynLib(secsse, .registration = TRUE)
