# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

eval_cpp <- function(rhs, ances, states, forTime, lambdas, mus, Q, method, atol, rtol, is_complete_tree, num_steps) {
    .Call(`_secsse_eval_cpp`, rhs, ances, states, forTime, lambdas, mus, Q, method, atol, rtol, is_complete_tree, num_steps)
}

calc_ll_cpp <- function(rhs, ances, states, forTime, lambdas, mus, Q, method, atol, rtol, is_complete_tree, see_states, use_normalization) {
    .Call(`_secsse_calc_ll_cpp`, rhs, ances, states, forTime, lambdas, mus, Q, method, atol, rtol, is_complete_tree, see_states, use_normalization)
}

ct_condition_cpp <- function(rhs, state, t, lambdas, mus, Q, method, atol, rtol, use_normalization) {
    .Call(`_secsse_ct_condition_cpp`, rhs, state, t, lambdas, mus, Q, method, atol, rtol, use_normalization)
}

secsse_sim_cpp <- function(m_R, lambdas_R, q_R, max_time, max_species, max_species_extant, min_species, init_state_probs, condition, num_concealed_states, non_extinction, verbose, max_tries, seed, conditioning_vec, return_tree_size_hist, start_at_crown) {
    .Call(`_secsse_secsse_sim_cpp`, m_R, lambdas_R, q_R, max_time, max_species, max_species_extant, min_species, init_state_probs, condition, num_concealed_states, non_extinction, verbose, max_tries, seed, conditioning_vec, return_tree_size_hist, start_at_crown)
}

calc_ll_single_branch_cpp <- function(rhs, states, forTime, lambdas, mus, Q, method, atol, rtol, see_states, use_normalization) {
    .Call(`_secsse_calc_ll_single_branch_cpp`, rhs, states, forTime, lambdas, mus, Q, method, atol, rtol, see_states, use_normalization)
}

